#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2024 StrayWarrior <i@straywarrior.com>

"""
Models for long article categories.
"""


import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.linear_model import LogisticRegression, LinearRegression
from sklearn.metrics import accuracy_score, confusion_matrix, classification_report
from sklearn.metrics import mean_squared_error, r2_score
import statsmodels.api as sm

class CategoryRegressionV1:
    def __init__(self):
        self.features = [
            #'ViewCountRate',
            'CateOddities', 'CateFamily', 'CateHeartwarm',
            'CateHistory', 'CateHealth', 'CateLifeKnowledge', 'CateGossip',
            'CatePolitics', 'CateMilitary'
        ]
        self.category_name_map = {
            '奇闻趣事': 'CateOddities',
            '历史人物': 'CateHistory',
            '家长里短': 'CateFamily',
            '温情故事': 'CateHeartwarm',
            '健康养生': 'CateHealth',
            '生活知识': 'CateLifeKnowledge',
            '名人八卦': 'CateGossip',
            '政治新闻': 'CatePolitics',
            '军事新闻': 'CateMilitary',
        }
        self.reverse_category_name_map = {
            v: k for k, v in self.category_name_map.items()
        }

    def preprocess_data(self, df):
        for cate in self.category_name_map:
            colname = self.category_name_map[cate]
            df[colname] = df['category'] == cate
            df[colname] = df[colname].astype(int)

        df['ClassY'] = df['read_avg_rate'] > 1
        df['RegressionY'] = df['read_avg_rate']
        return df

    def build_and_print(self, df, account_name):
        if account_name is not None:
            sub_df = df[df['account_name'] == account_name]  
        else:
            sub_df = df
        if len(sub_df) < 5:
            return
        sample_count = len(sub_df)
        params, t_stats, p_values = self.run_ols_linear_regression(sub_df)
        row = f'{account_name}\t{sample_count}'
        for param, p_value in zip(params, p_values):
            row += f'\t{param:.3f}\t{p_value:.3f}'
        print(row)

    def build(self, df):
        p_value_column_names = '\t'.join([name + "\tp-" + name for name in
                                          ['bias'] + self.features])
        print('account\tsamples\t{}'.format(p_value_column_names))
        self.build_and_print(df, None)
        for account_name in df['account_name'].unique():
            self.build_and_print(df, account_name)

    def get_param_names(self):
        return ['bias'] + self.features

    def run_ols_linear_regression(self, df):
        X = df[self.features]  # 特征列
        y = df['RegressionY']  # 目标变量
        X = sm.add_constant(X)

        model = sm.OLS(y, X).fit()

        params = model.params
        t_stats = model.tvalues
        p_values = model.pvalues
        conf_int = model.conf_int()

        return params, t_stats, p_values

def main():
    df = pd.read_excel('20241101_read_rate_samples.xlsx')  # 如果数据来自CSV文件
    df['read_avg'] = df['阅读均值']
    df['read_avg_rate'] = df['阅读倍数']
    df['dt'] = df['日期']
    df['similarity'] = df['Similarity']
    filter_condition = 'read_avg > 500 ' \
        'and read_avg_rate > 0 and read_avg_rate < 3 ' \
        'and dt > 20240914 and similarity > 0' 
    df = df.query(filter_condition).copy()

    m_cate = CategoryRegressionV1()

    df = m_cate.preprocess_data(df)
    m_cate.build(df)


if __name__ == '__main__':
    main()
